# region imports

from dos_plots import *
from matplotlib import gridspec
from investigate_bands import *
from clarissa_utils import home_dir

# endregion

# region paths
home = home_dir()
cwd = "rem_n147_n228"
folder = "1811_ldos"
wd = os.path.join(home_dir(), cwd, folder)
os.makedirs(wd, exist_ok=True)
brg_path = os.path.join(home_dir(), cwd, folder, "brg", "dos_result_dw1e-03.json")

# endregion

# region functions

def ldos_plot(brg_path, cbar=True):
    w_min = 0.05
    dos_result_brg = load(brg_path)
    w_bins = dos_result_brg.w_bins
    # Setup data.
    data = assemble_dos_data(dos_result_brg, "ldos3d", fltr=lambda x: gf1d(x, 5))
    idx_min = np.argmin(abs(w_bins - w_min))
    w_bins = w_bins[idx_min:]
    data = data[idx_min:, :]
    x = np.linspace(-0.5, 0.5, data.shape[1])
    X, Y = np.meshgrid(x, w_bins)
    # plt.pcolor(X, Y, data, cmap='jet')
    plt.pcolor(X, Y, data, cmap='jet', vmin=0.8, vmax=2.0)  # inject scale here (depends on selected n_highs!)
    plt.xlim(-0.5, 0.5)
    plt.ylim(min(w_bins), max(w_bins))
    plt.xlabel("x [a]")
    # Add text.
    plt.text(-0.25, 1.43, "$n_{\mathregular{low}}$")
    plt.text(0.25, 1.43, "$n_{\mathregular{high}}$")
    if not cbar:
        return
    cbar = plt.colorbar(ticks=[0.8, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0])
    cbar.set_label("LDOS$_{\mathregular{rel}}$", labelpad=-8)
    cbar.ax.set_yticklabels(['0.8', '1.0', '1.2', '1.4', '1.6', '1.8', '$\geq$ 2.0'])
    return x, w_bins, data

def relative_ldos_plot(brg_path, lmbd_res=100, space_res=32, mark_bg=True):
    ldos_input = RemDosFileInput(brg_path, lambda x: gf1d(x, 5))
    lmbds = np.linspace(500, 2200, lmbd_res)
    lmbs = np.array(hofmann2016()["lmbs"])
    ldos_input.open()
    # Calculate results.
    l21, l31 = [], []
    for i in range(len(lmbds)):
        corr = ldos_correction(lmbs, ldos_input, lmbds[i], space_res)
        l21.append(np.mean([item[0, 1] for item in corr]))
        l31.append(np.mean([item[0, 2] for item in corr]))
    l21 = np.array(l21)
    l31 = np.array(l31)
    plt.xlabel("$\lambda_D$ [nm]")
    plt.ylabel("$\\bar{\mathregular{LDOS}}_{\mathregular{rel}}")
    ln21, = plt.plot(lmbds, l21, label="SPE21 (loss)")
    ln31, = plt.plot(lmbds, l31, label="SPE31 (UC)")
    ratio = l31 / l21
    #ln3121, = plt.plot(lmb_designs, ratio, label="ratio")
    ax = plt.gca()
    lmb_opt = lmbds[ratio == np.max(ratio)][0]
    ax.axvline(x=lmb_opt, ls='--', color='black', alpha=0.5)
    ax.legend(*zip(*[(il, il.get_label()) for il in [ln31, ln21]]), loc=3)
    return lmbds, l31, l21, ratio


def get_epsilon(wd):
    wd_eval = os.path.join(wd)
    os.makedirs(wd_eval, exist_ok=True)
    filename_save = os.path.join(wd_eval, "epsilons.txt")

    ldos_result = load(os.path.join(wd, "brg", "dos_result_dw1e-03.json"))
    epsilons = np.array(ldos_result.epsilon)
    np.savetxt(filename_save, epsilons)

def band_gap_edges_from_file(brg_path):
    """
    Get the omega values of the edges of the band gap(s).
    :param n_low: refractive index of low index materials
    :param n_high: refractive index of high index materials
    :return: vector of band edge points
    """

    result = load(brg_path)
    # Derive band gaps.
    return np.array(result.ws['te'][0])

# endregion

# Generate (and save) the figure.

###########CHECK OUT THESE FUNCTIONS IF WANT TO SIMULATE BANDSTRUCTURE
#PBG_edges = band_gap_edges_from_file(brg_path)
#result = load(brg_path)
#mpb_result = build_mpb_result_object(wd, args)


#endregion

get_epsilon(wd)


# relative dos for SPE31, SPE21 and their reatio SPE31/SPE21 against lmbd for lmb=1523nm
dos_result=np.array(relative_ldos_plot(brg_path)).transpose()
plt.savefig(os.path.join(wd, "dos_lmbd_spe31_spe21_ratio.png"), dpi=300)
#plt.show()
file_name = "dos_lmbd_spe31_spe21_ratio.txt" #"{}.txt".format(name)
file_path = os.path.join(wd, file_name) if wd is not None else file_name
np.savetxt(file_path, dos_result)

# relative ldos plot
ldos_result=ldos_plot(brg_path)
origin_export2d(ldos_result[1], ldos_result[0], ldos_result[2], "ldos_rel", wd)
plt.savefig(os.path.join(wd, "ldos_rel.png"), dpi=300)
#plt.show()


#Stuff to remember...
#mpb_result = build_mpb_result_object(brg_path, sym = True, )
#MpbResult
#edges = band_gap_edges_from_file(brg_path)