
from dos_plots import *
from clarissa_utils import *

from dos_utils import *
import matplotlib.pyplot as plt

# region things to remember

# parameter_names[0]='ldos_input'
# parameter_names[1]='lmbd'
# parameter_names[2]='lmb'
# parameter_names[3]='ang'
# parameter_names[4]='irr'

# rem_results.abs_brg, .shape = (x,1)
# rem_results.abs_ref, .shape = (x,1)
# rem_results.lum_brg, .shape = (x,6)
# rem_results.lum_ref, .shape = (x,6)
# rem_results.get_lum_rel(), .shape = (x,6)
# rem_results.get_ucqy_brg(), .shape = (x,1)
# rem_results.get_ucqy_ref(), .shape = (x,1)
# rem_results.get_qy_brg(), .shape = (x,6)
# rem_results.get_qy_ref(), .shape = (x,6)

# endregion

# region functions
## TODO graphs are not used so far, need to be adapted to 1D if want to be used

def eval_rem_matrix(matrix, name, x, y, x_label, y_label, wd=None, transition=None, plt_show=False, transpose=True):
    """
    reshape, plot and export rem result data 
    (rem_result.abs_ref, rem_result.lum_ref, rem_result.abs_brg, rem_result.lum_brg, rem_result.lum_rel)   

    :param matrix: matrix to evaluate, 2D  rem_result.abs_ref and rem_result.abs_brg
            or 3D rem_result.lum_ref, rem_result.lum_brg, rem_result.lum_rel 
    :param name: name of parameter to be evaluated in z-axis
    :param x: x-axis for meshgrid, e.g. rem_result.param_values[1]
    :param y: y-axis for meshgrid
    :param x_label: x-axis label, e.g. lmbd [nm]
    :param x_label: y-axis label
    :param transition: transition to be evaluated, default is None for 2D matrix.
            if matrix is 3D the transition needs to be specified, 0 for 1558nm, 1 for 984 nm...
    :param folder_save: folder in which returned data is saved
    :param plt_show: default is True, if plt_show is False, generated graphs are not displayed
    :return: 2D matrix with rem result data in origin_export_2D format and the corersponding graph
    """

    X, Y = np.meshgrid(y, x)
    if transition is None:
        matrix=matrix.reshape(X.shape[0], X.shape[1])
    else:
        matrix_shape = matrix.reshape(X.shape[0], X.shape[1], -1)
        matrix = matrix_shape[:, :, transition]

    if wd is not None:
        if transpose is True:
            origin_export2d(y, x, np.transpose(matrix), name="{}".format(name), wd=wd)
        else:
            origin_export2d(x, y, matrix, name="{}".format(name), wd=wd)
    if plt_show is True:
        plt.pcolor(Y, X, matrix)
        plt.title("{}".format(name))
        plt.xlabel("{}".format(x_label))
        plt.ylabel("{}".format(y_label))
        cb = plt.colorbar(label="{}".format(name), spacing='proportional')
        plt.savefig(os.path.join(wd, "{}".format(name) + ".png"), dpi=300)
        plt.show()
        cb.remove()


def eval_rem_matrix_times_100(matrix, name, x, y, x_label, y_label, wd=None, transition=None, plt_show=False, transpose=True):
    """
    reshape, plot and export rem result data 
    (rem_result.abs_ref, rem_result.lum_ref, rem_result.abs_brg, rem_result.lum_brg, rem_result.lum_rel)   

    :param matrix: matrix to evaluate, 2D  rem_result.abs_ref and rem_result.abs_brg
            or 3D rem_result.lum_ref, rem_result.lum_brg, rem_result.lum_rel 
    :param name: name of parameter to be evaluated in z-axis
    :param x: x-axis for meshgrid, e.g. rem_result.param_values[1]
    :param y: y-axis for meshgrid
    :param x_label: x-axis label, e.g. lmbd [nm]
    :param x_label: y-axis label
    :param transition: transition to be evaluated, default is None for 2D matrix.
            if matrix is 3D the transition needs to be specified, 0 for 1558nm, 1 for 984 nm...
    :param folder_save: folder in which returned data is saved
    :param plt_show: default is True, if plt_show is False, generated graphs are not displayed
    :return: 2D matrix with rem result data in origin_export_2D format and the corersponding graph
    """

    X, Y = np.meshgrid(y, x)
    if transition is None:
        matrix=matrix.reshape(X.shape[0], X.shape[1])*100
    else:
        matrix_shape = matrix.reshape(X.shape[0], X.shape[1], -1)
        matrix = matrix_shape[:, :, transition]

    if wd is not None:
        if transpose is True:
            origin_export2d(y, x, np.transpose(matrix), name="{}".format(name), wd=wd)
        else:
            origin_export2d(x, y, matrix, name="{}".format(name), wd=wd)
    if plt_show is True:
        plt.pcolor(Y, X, matrix)
        plt.title("{}".format(name))
        plt.xlabel("{}".format(x_label))
        plt.ylabel("{}".format(y_label))
        cb = plt.colorbar(label="{}".format(name), spacing='proportional')
        plt.savefig(os.path.join(wd, "{}".format(name) + ".png"), dpi=300)
        plt.show()
        cb.remove()

def get_lum_rel_spe41_plus_spe62(rem_result, name, x, y, x_label, y_label, wd = None, plt_show = False, transpose=True):

    X, Y = np.meshgrid(y, x)

    lum_brg=rem_result.lum_brg
    lum_ref=rem_result.lum_ref

    lum_brg_shape = lum_brg.reshape(X.shape[0], X.shape[1], -1)
    lum_ref_shape = lum_ref.reshape(X.shape[0], X.shape[1], -1)

    lum_brg41 = lum_brg_shape[:, :, 2]
    lum_brg62 = lum_brg_shape[:, :, 5]
    lum_ref41 = lum_ref_shape[:, :, 2]
    lum_ref62 = lum_ref_shape[:, :, 5]

    matrix=(lum_brg41+lum_brg62)/(lum_ref41+lum_ref62)

    if wd is not None:

        if transpose is True:
            origin_export2d(y, x, np.transpose(matrix), name="{}".format(name), wd=wd)
        else:
            origin_export2d(x, y, matrix, name="{}".format(name), wd=wd)
    if plt_show is True:
        plt.pcolor(Y, X, matrix)
        plt.title("{}".format(name))
        plt.xlabel("{}".format(x_label))
        plt.ylabel("{}".format(y_label))
        cb = plt.colorbar(label="{}".format(name), spacing='proportional')
        plt.savefig(os.path.join(wd, "{}".format(name) + ".png"), dpi=300)
        plt.show()
        cb.remove()

def eval_rem(param_values_x, param_values_y, labelx, labely, wd, plt_show=False, transpose=True, lum_rel=True, lum_brg=True, lum_ref=True, abs=False, ucqy = True):

    rem_result = load(os.path.join(wd, "rem_result.json"))
    x = rem_result.param_values[param_values_x]
    y = rem_result.param_values[param_values_y]

    if lum_rel is True:
        eval_rem_matrix(rem_result.get_lum_rel(), "lum_rel_1558nm", x, y, labelx, labely, wd, 0, plt_show, transpose)
        eval_rem_matrix(rem_result.get_lum_rel(), "lum_rel_984nm", x, y, labelx, labely, wd, 1, plt_show, transpose)
        eval_rem_matrix(rem_result.get_lum_rel(), "lum_rel_814nm", x, y, labelx, labely, wd, 2, plt_show, transpose)
        eval_rem_matrix(rem_result.get_lum_rel(), "lum_rel_660nm", x, y, labelx, labely, wd, 3, plt_show, transpose)
        eval_rem_matrix(rem_result.get_lum_rel(), "lum_rel_536nm", x, y, labelx, labely, wd, 4, plt_show, transpose)
        eval_rem_matrix(rem_result.get_lum_rel(), "lum_rel_817nm", x, y, labelx, labely, wd, 5, plt_show, transpose)
        get_lum_rel_spe41_plus_spe62(rem_result, "lum_rel_814nm_817nm", x, y, labelx, labely, wd, plt_show, transpose)

    if lum_brg is True:
        eval_rem_matrix(rem_result.lum_brg, "lum_brg_1558nm", x, y, labelx, labely, wd, 0, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_brg, "lum_brg_984nm", x, y, labelx, labely, wd, 1, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_brg, "lum_brg_814nm", x, y, labelx, labely, wd, 2, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_brg, "lum_brg_660nm", x, y, labelx, labely, wd, 3, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_brg, "lum_brg_536nm", x, y, labelx, labely, wd, 4, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_brg, "lum_brg_817nm", x, y, labelx, labely, wd, 5, plt_show, transpose)

    if lum_ref is True:
        eval_rem_matrix(rem_result.lum_ref, "lum_ref_1558nm", x, y, labelx, labely, wd, 0, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_ref, "lum_ref_984nm", x, y, labelx, labely, wd, 1, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_ref, "lum_ref_814nm", x, y, labelx, labely, wd, 2, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_ref, "lum_ref_660nm", x, y, labelx, labely, wd, 3, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_ref, "lum_ref_536nm", x, y, labelx, labely, wd, 4, plt_show, transpose)
        eval_rem_matrix(rem_result.lum_ref, "lum_ref_817nm", x, y, labelx, labely, wd, 5, plt_show, transpose)

    if abs is True:
        eval_rem_matrix(rem_result.abs_ref, "abs_ref", x, y, labelx, labely, wd, transition=None, plt_show=plt_show, transpose=transpose)
        eval_rem_matrix(rem_result.abs_brg, "abs_brg", x, y, labelx, labely, wd, transition=None, plt_show=plt_show, transpose=transpose)

    if ucqy is True:
        eval_rem_matrix_times_100(rem_result.get_ucqy_ref(), "ucqy_ref", x, y, labelx, labely, wd, transition=None,plt_show=plt_show, transpose=transpose)
        eval_rem_matrix_times_100(rem_result.get_ucqy_brg(), "ucqy_brg", x, y, labelx, labely, wd, transition=None, plt_show=plt_show, transpose=transpose)


# endregion

# region generate evaluation

path = "rem_n147_n228/1901_rem/"
cwd = os.path.join(home_dir(), path)
os.makedirs(cwd, exist_ok=True)

def eval_lmbd(cwd,name):
    # FIG 3B & 4A
    folders = [
        ("rem_lmbd_irr1523"), ("rem_lmbd_irr1523_noldos"), ("rem_lmbd_irr1523_nobs"),
        ("rem_lmbd_irr1sun"), ("rem_lmbd_irr1sun_noldos"), ("rem_lmbd_irr1sun_nobs")]

    labelx = "design wavelength [nm]"  # 1
    labely = "excitation wavelength [nm]"  # 2

    for folder in folders:
        wd = os.path.join(home_dir(), cwd, folder + "_" + name)
        os.makedirs(wd, exist_ok=True)
        eval_rem(param_values_x=1, param_values_y=2, labelx=labelx, labely=labely, wd=wd, plt_show=False, transpose=False,
                 lum_rel=True, lum_brg=True, lum_ref=True)  # , abs=True)

def eval_lmb(cwd, name):
     # FIG 4C
    folders = [("rem_lmbd_lmb_irr1500"), ("rem_lmbd_lmb_irr1560")]

    labelx = "design wavelength [nm]"  # 1
    labely = "excitation wavelength [nm]"  # 2

    for folder in folders:
        wd = os.path.join(home_dir(), cwd, folder + "_" + name)
        os.makedirs(wd, exist_ok=True)
        eval_rem(param_values_x=1, param_values_y=2, labelx=labelx, labely=labely, wd=wd, plt_show=False,
                 transpose=True, lum_rel=True, lum_brg=True, lum_ref=True)  # , abs=True)

def eval_irr(cwd, name):
    # FIG 4B
    folders = [("rem_lmbd_irr"), ("rem_lmbd_irr_nobs"), ("rem_lmbd_irr_noldos")]

    labelx = "design wavelength [nm]"  # 1
    labely = "irradiance [W/m2]"  # 4

    for folder in folders:
        wd = os.path.join(home_dir(), cwd, folder + "_" + name)
        os.makedirs(wd, exist_ok=True)
        eval_rem(param_values_x=1, param_values_y=4, labelx=labelx, labely=labely, wd=wd, plt_show=False,
                 transpose=True, lum_rel=True, lum_brg=True, lum_ref=True)  # , abs=True)

# endregion

for name in [("ideal"),("mc")]:
    eval_lmbd(cwd, name)
    eval_lmb(cwd, name)
    eval_irr(cwd, name)