# region imports
#
from irr_core import *
from tmmpy_mc import *
from tmmpy import *
import matplotlib.pyplot as plt
from rem_core import *
from bragg_utils import *
from clarissa_utils import home_dir

# endregion

cwd = "rem_n147_n228"
folder = "1901_u_mc"
# folder = "1811_u_mc_std_map"
wd = os.path.join(home_dir(), cwd, folder)
os.makedirs(wd, exist_ok=True)


# endregion

# region input files

# region function for plots
def plot_irr_result(wd, name, label=None, lmbd=None, lmb=None, theta=None):
    wd = os.path.join(home_dir(), cwd, folder, name)
    irr_file = os.path.join(wd, "irr_result.json")  #
    irr_result = load(irr_file)
    # parameters read out from irr file
    # lmbds = irr_result.param_values[0] if lmbd is None else lmbd
    # lmbs = irr_result.param_values[1] if lmb is None else lmb
    # thetas = irr_result.param_values[2] if theta is None else theta
    # irr_result0 = select_value(irr_result, {"lmbd": lmbds[0], "lmb": lmbs[0], "ang": thetas[0]})[0]

    lmbds = (irr_result.param_values[0])[0] if lmbd is None else lmbd
    lmbs = (irr_result.param_values[1])[0] if lmb is None else lmb
    thetas = (irr_result.param_values[2])[0] if theta is None else theta
    irr_result0 = select_value(irr_result, {"lmbd": lmbds, "lmb": lmbs, "ang": thetas})[0]

    label = name if label is None else label
    plt.plot(irr_result0, label=label)
    plt.xlabel('points')
    plt.ylabel('u_rel')
    plt.legend()

    file_name = "x_u_rel_{}.txt".format(name)
    file_path = os.path.join(wd, file_name)
    np.savetxt(file_path, irr_result0)

    # file_path = os.path.join(wd, "u_rel_std_x_{}".format(std_high) + ".png")
    # file_path = os.path.join(wd, "{}".format(name)+".txt")
    # np.savetxt(file_path, irr_result0)


def calc_mean_rel_irrs_fileinput(wd, name, lmb=None, ang=None, save=True, plot=True):
    """
    Wrapper for easy scanning over lmbds.
    :param lmb: excitation wavelength
    :param ang: angle of incidence (in rad)
    :param irr_result: IrrResult object (containing only the active layers of the bragg stack)
    :return: [design wavelength [nm], vector of mean relative irradiance across the active layers against the 
    """
    irr_file = os.path.join(wd, name, "irr_result.json")  #
    irr_result = load(irr_file)

    lmbds = irr_result.param_values[0]
    lmbs = irr_result.param_values[1]
    thetas = irr_result.param_values[2]
    lmb = lmbs[0] if lmb is None else lmb
    theta = thetas[0] if ang is None else ang

    design_res = len(lmbds)
    irr_enhancements = np.zeros(shape=(4, design_res))
    for i in range(design_res):
        irr_brg = select_value(irr_result, {"lmbd": lmbds[i], "lmb": lmb, "ang": theta})[0]
        irr_ref = select_value(irr_result, {"lmbd": lmbds[i], "lmb": lmb, "ang": theta})[1]
        sum_irr_brg = sum(irr_brg)
        sum_irr_ref = irr_ref * irr_brg.size
        irr_enhancements[:, i] = [lmbds[i], sum_irr_brg, sum_irr_ref, sum_irr_brg / sum_irr_ref]
    if save is True:
        z = np.zeros(shape=(len(irr_enhancements[0]), 2))
        z[:, 0] = irr_enhancements[0]
        z[:, 1] = irr_enhancements[3]
        file_name = "lmbd_u_rel_{}.txt".format(name)
        file_path = os.path.join(wd, name, file_name)
        np.savetxt(file_path, z)
    if plot is True:
        plt.plot(irr_enhancements[0], irr_enhancements[3], label=name + "_rel")
        plt.xlabel('lmbd [nm]')
        plt.ylabel('u_rel')
        plt.legend()

    return irr_enhancements


# endregion

# region calculation

# plot_irr_result(cwd, folder, "calc_irr_data", "std_0")
for name in [("ideal_lmbds"), ("mc_lmbds")]:
    irr_rel = calc_mean_rel_irrs_fileinput(wd, name)
plt.savefig(os.path.join(wd, "lmbd_u_rel_std.png"), dpi=300)
plt.show()

plot_irr_result(wd, "ideal_lmbs", None, lmbd=1888, lmb=1523, theta=None)
plot_irr_result(wd, "mc_lmbs", None, lmbd=1888, lmb=1523, theta=None)
plt.savefig(os.path.join(wd, "x_u_rel.png"), dpi=300)
plt.show()


# plt.savefig(os.path.join(wd, "{}".format(std_high) + ".png"), dpi=300)
# plt.savefig("2.png", dpi=300)
# plt.show()

# plt.savefig(os.path.join(cwd, folder, "u_rel.png"), dpi=300)

# endregion

# region legacy

def plot_mean_irrs(irr_enhancements, label):
    plt.plot(irr_enhancements[0], irr_enhancements[1], label=label + "_brg")
    plt.plot(irr_enhancements[0], irr_enhancements[2], label=label + "_ref")
    plt.xlabel('lmbd [nm]')
    plt.ylabel('u [a.u.]')
    plt.legend()


def plot_mean_rel_irrs(irr_enhancements, label):
    plt.plot(irr_enhancements[0], irr_enhancements[3], label=label + "_rel")
    plt.xlabel('lmbd [nm]')
    plt.ylabel('u_rel')
    plt.legend()
