# region inputs

from rem_core import *
from clarissa_utils import *

# endregion

# region paths

cwd = "rem_n147_n228"
ldos_folder = "1811_ldos"
irr_folder = "1901_u_mc" #irradiance input folder
folder = "1901_rem"
# endregion

# region parameters

# define resolution for rem
res_rem = 32
cores_rem = 30

#irradiances from experiment (area of FWHM, irradiance of FWHM (total irradiance scaled with
# factor 0.50006(from 2D integral over gaussian in polar coord. from 0 to +FWHM/2)
# and scaned with cos(4/180*pi) for tild of samples of 4° in integrating sphere
irr_1523 = np.array([14809])  # irradinace at 1523 nm and 10 mW in experiment from powerscan
irr_1560 = np.array([13781])  # irradiance at 1560 nm and 10 mW
irr_1500 = np.array([15673])  # irradiance at 1500 nm and 10 mW
irr_1sun = np.array([30])  # irradiance at 1 sun
# irradiances list
irrs = np.array(list(np.arange(10, 1000, 10)) + list(np.arange(1000, 5000, 100)) + list(np.arange(5000, 50100, 1000)))
# design wavelength of sample for power scan
lmbd_1888 = np.array([1888])
lmb_1523 = np.array([1523])
# min and max lmbds of groups in experiment
#lmbds_small = np.array([1784, 1805, 1835, 1865, 1880, 1890, 1910, 1930, 1985, 2005])

# endregion

def run_rem_calculations(name):
    # define/create output folder
    wd = os.path.join(home_dir(), cwd, folder)
    os.makedirs(wd, exist_ok=True)
    # region inputs

    #path input files
    dos_brg_file = os.path.join(home_dir(), cwd, ldos_folder, "brg", "dos_result_dw1e-03.json")
    irr_brg_file_lmbds = os.path.join(home_dir(), cwd, irr_folder, name + "_lmbds", "irr_result.json")
    irr_brg_file_lmbs = os.path.join(home_dir(), cwd, irr_folder, name + "_lmbs", "irr_result.json")
    #load input files
    dos_input = RemDosFileInput(dos_brg_file, fltr=lambda x: gf1d(x, 5))
    abs_er = read_absorption_spectrum("abs_er")
    # irr file is read out in loops directly

    # endregion

    # region FIG 2B & 3A (lmbd scan)
    #parameters:
    # lmbds=(900,2100,x)
    # lmbs=1523nm fix
    # ang: fixed incident angle ang=8° like in experiment
    # irr: fixed irradiance of 1523 nm excitation and of 1 sun
    # additionally: noldos and nobs calculation!

    ## 3B & 4A) calculation with irradiance at lmb=1523 nm


#    irr_input = load(irr_brg_file_lmbs)

    # load irr file and read our parameters out from irr file
    irr_brg_file = irr_brg_file_lmbds
    irr_input = load(irr_brg_file)
    lmbds = irr_input.param_values[0]
    lmbs = irr_input.param_values[1]
    thetas = irr_input.param_values[2]

    calc= "rem_lmbd_irr1523"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbds, irr_1523, angs=thetas, ldos_input=[dos_input])
    params["lmb"]=lmb_1523
    rem_input = RemInput(hofmann2016(), RemIrrFileInput(irr_brg_file), params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # 3B) corresponding to 3A, same calculation with no ldos impact
    calc= "rem_lmbd_irr1523_noldos"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbds, irr_1523, angs=thetas, ldos_input=None)
    params["lmb"]=lmb_1523
    rem_input = RemInput(hofmann2016(), RemIrrFileInput(irr_brg_file), params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # 3C) corresponding to 3A, same calculation with no irradiance enhancement impact ("nobs" for no bragg stack)
    calc= "rem_lmbd_irr1523_nobs"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbds, irr_1523, angs=thetas, ldos_input=[dos_input])
    params["lmb"]=lmb_1523
    rem_input = RemInput(params=hofmann2016(), irr_input=None, param_map=params, abs_spec=abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    ## calculation at 1 sun irradiance

    # 3A) irradiance at 1 sun
    calc = "rem_lmbd_irr1sun"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbds, irr_1sun, angs=thetas, ldos_input=[dos_input])
    params["lmb"] = lmb_1523
    rem_input = RemInput(hofmann2016(), RemIrrFileInput(irr_brg_file), params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # 3B) corresponding to 3A, same calculation with no ldos impact
    calc= "rem_lmbd_irr1sun_noldos"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbds, irr_1sun, angs=thetas, ldos_input=None)
    params["lmb"]= lmb_1523
    rem_input = RemInput(hofmann2016(), RemIrrFileInput(irr_brg_file), params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # 3C) corresponding to 3A, same calculation with no irradiance enhancement impact ("nobs" for no bragg stack)
    calc= "rem_lmbd_irr1sun_nobs"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbds, irr_1sun, angs=thetas, ldos_input=[dos_input])
    params["lmb"]= lmb_1523
    rem_input = RemInput(params=hofmann2016(), irr_input=None, param_map=params, abs_spec=abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # endregion

    # region FIG 4B (irradiance scan)

    # rem calculation: irradiance scan with fixed ang=8°, fixed lmb=1523 nm, fixed lmbd=1888nm

    # load irr file and read our parameters out from irr file
    irr_brg_file = irr_brg_file_lmbs
    irr_input = load(irr_brg_file)
    lmbds = irr_input.param_values[0]
    lmbs = irr_input.param_values[1]
    thetas = irr_input.param_values[2]

    calc = "rem_lmbd_irr"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbd_1888, irrs, angs=thetas, ldos_input=[dos_input])
    params["lmb"] = lmb_1523
    rem_input = RemInput(hofmann2016(), RemIrrFileInput(irr_brg_file), params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # noldos, same calculation with no ldos impact
    calc = "rem_lmbd_irr_noldos"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbd_1888, irrs, angs=thetas, ldos_input=None)
    params["lmb"] = lmb_1523
    rem_input = RemInput(hofmann2016(), RemIrrFileInput(irr_brg_file), params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    #nobs, same calculation with no irradiance enhancement impact ("nobs" for no bragg stack)
    calc = "rem_lmbd_irr_nobs"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbd_1888, irrs, angs=thetas, ldos_input=[dos_input])
    params["lmb"] = lmb_1523
    rem_input = RemInput(hofmann2016(), None, params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # endregion)

    # region FIG 4C (lmb scan)

    irr_brg_file = irr_brg_file_lmbs
    irr_input = load(irr_brg_file)
    lmbds = irr_input.param_values[0]
    lmbs = irr_input.param_values[1]
    thetas = irr_input.param_values[2]

    # 4C_1) irradiance at lmb=1500 nm
    calc= "rem_lmbd_lmb_irr1500"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbds, irr_1500, angs=thetas, ldos_input=[dos_input])
    params["lmb"]=lmbs
    rem_input = RemInput(hofmann2016(), RemIrrFileInput(irr_brg_file), params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # 4C_2) irradiance at lmb=1560 nm
    calc= "rem_lmbd_lmb_irr1560"
    calc = "_".join([calc, name])
    params = default_rem_params(lmbds, irr_1560, angs=thetas, ldos_input=[dos_input])
    params["lmb"]=lmbs
    rem_input = RemInput(hofmann2016(), RemIrrFileInput(irr_brg_file), params, abs_er)
    run_rem(calc, rem_input, wd=wd, res=res_rem, cores=cores_rem)

    # endregion


for name in [("ideal"), ("mc")]: # if needed folder in folder_in for irradiance input
    run_rem_calculations(name)
