# region imports
from operator import add
import matplotlib.pyplot as plt
from bragg_objects import *
from irr_core import *
from tmmpy_mc import *
from tmmpy import *
from rem_core import *
from bragg_utils import *
from clarissa_utils import home_dir

# endregion

# region things to remember

# reshape matrix for varied axis x,y in simulation run
# irr_result.param_names[0]='lmbd'
# irr_result.param_names[1]='lmb'
# irr_result.param_names[2]='ang'

# x = irr_result.param_values[1]
# y = irr_result.param_values[2]#*180/np.pi
# X, Y = np.meshgrid(y, x)
# matrix = irr_result.irrs[0]

# endregion

# region paths
cwd = "rem_n147_n228"
# folder = "1811_u_mc"
folder = "1901_u_mc"
wd = os.path.join(home_dir(), cwd, folder)
os.makedirs(wd, exist_ok=True)
# endregion

# region define varied parameters
itr = 1000
res = 32
std = 1

std_low = 4.15
std_high = 1.53

thetas = np.arange(4, 90, 90) / 180 * np.pi
lmbs = np.array(list(np.arange(1375, 1521, 5)) + list(np.arange(1523, 1524, 1)) + list(np.arange(1525, 1651, 5)))
lmbds_small = np.array(
    list(np.arange(1780, 1785, 4)) + list(np.arange(1785, 1886, 5)) + list(np.arange(1888, 1889, 1)) + list(
        np.arange(1890, 2006, 5)))

lmbs_small = np.arange(1523, 1524, 1)
lmbds = np.arange(500, 2501, 5)

# endregion

# region construct stacks

pmma_np = read_material("1711_pmmanp")
tio2_ald = read_material("1711_tio2")
bs = BraggStack(low=pmma_np, high=tio2_ald, lmbd=lmbds[0], num_al=4, lmb8=None)

# endregion

# region locally resolved energy density in all layers

# scan_irr_data_s = scan_irr(bs=bs, lmb=lmbs[0], pol="s", ang=theta, res=res)
# scan_irr_data_p = scan_irr(bs=bs, lmb=lmbs[0], pol="p", ang=theta, res=res)
# scan_irr_data = (np.array(scan_irr_data_s[1]) + np.array(scan_irr_data_p[1])) / 2

# endregion

# region IrrResult object ideal
params = default_irr_params(lmbds=lmbds_small, angs=thetas, lmbs=lmbs)
irr_result = run_irr_calculation(name="ideal_lmbs", bs=bs, param_map=params, wd=wd, calc=calc_irr_data, res=res)

params = default_irr_params(lmbds=lmbds, angs=thetas, lmbs=lmbs_small)
irr_result = run_irr_calculation(name="ideal_lmbds", bs=bs, param_map=params, wd=wd, calc=calc_irr_data, res=res)


# endregion

# region IrrResult object mc functions

def run_irr_calculation_mc(name, bs, param_map, res=None, wd=None, cores=None, calc=None):
    """
    Main entry point of irradiance calculation.
    :param name: name of the calculation, a folder with this name is created (and cleaned!) on execution
    :param bs: bragg stack object ("BraggStack")
    :param param_map: map of parameters, should contain design wavelengths "lmbd", excitation wavelengths "lmb" and
    incident angles "ang"
    :param res: number of points evaluated per active layer
    :param wd: working directory in which folder with result files is created
    :param cores: number of cores to utilize, default behaviour is to pick the number of cores available on the system
    :param calc: calculation function, default is "calc_irr_result"
    :return: irradiance result object ("IrrResult")
    """
    # Initialize default parameters.
    res = 16 if res is None else res  # minimum recommended resolution, for publications try higher
    calc = calc_irr_data if calc is None else calc
    cores = cores if cores is not None else multiprocessing.cpu_count()

    def build_info():
        info = {"name": name, "res": res, "bs": jsonpickle.encode(bs), "params": {}}
        # info = {"name": name, "res": res, "bs": jsonpickle.encode(bs), "std_map": jsonpickle.encode(std_map), "params": {}}
        for key in param_map:
            value = param_map[key]
            info["params"][key] = value.tolist() if isinstance(value, (np.ndarray, np.generic)) else value
        return info

    def build_slices(cwd):
        param_names = ["lmbd", "lmb", "ang"]
        param_values = [param_map[key].tolist() for key in param_names]
        slices = []
        for lmbd in param_map["lmbd"]:
            for lmb in param_map["lmb"]:
                for ang in param_map["ang"]:
                    slices.append({"lmbd": lmbd, "lmb": lmb, "ang": ang, "res": res, "bs": bs})
        return {"slices": slices, "param_names": param_names, "param_values": param_values}

    def build_result(cwd, slice_map, entries):
        irr_result = IrrResult(slice_map["param_names"], slice_map["param_values"], {'num_al': bs.num_al}, entries)
        with open(os.path.join(cwd, "irr_result.json"), 'w') as out_file:
            out_file.write(jsonpickle.encode(irr_result))
        return irr_result

    # Setup injections and dispatch.
    funcs = {"build_info": build_info, "build_slices": build_slices, "build_result": build_result, "calc": calc}
    return dispatch(name, "IRR", funcs, wd, cores)


def calc_irr_data_mc(args):
    """
    Calculation of irradiance data with layer thickness variation via the monte carlo.
    :param args: irradiance calculation parameters (see "run_irr_calculation_mc")
    :return: vector of relative irradiance in active layers
    """
    args["bs"].lmbd = args["lmbd"]
    #    pts, irr_brg, irr_ref = calc_irr(args["lmb"], args["ang"], args["bs"], args["res"])
    pts, irr_brg, irr_ref = calc_irr_mc(args["lmb"], args["ang"], args["bs"], args["res"], itr, std)

    return irr_brg, irr_ref


def calc_irr_mc(lmb, ang, bs, res=100, itr=1000, std=1):
    """
    Main entry point for calculation of the relative irradiance across the structure with monte carlo extension as used
    as input into the rem.
    :param lmb: excitation wavelength
    :param ang: angle of incidence (in rad)
    :param bs: bragg stack object ("BraggStack")
    :param res: spatial resolution per layer
    :param itr: number of monte carlo iterations
    :param std: standard deviation of the gaussian distribution from which the d_depths are draw
    :return: vector of relative irradiance at each point in the active layers, polarisation averaged
    """
    return calc_irr(lmb, ang, bs, res,
                    lambda lmb, ang, bs, res, pol: calc_irr_inner_mc(lmb, ang, bs, res, pol, itr, std))


def calc_irr_inner_mc(lmb, ang, bs, res, pol, itr=1000, std=1):
    """
    Calculate relative irradiance across the structure with monte carlo extension. Rather
    than calling the tmm interface of tmmpy, a semi-custom (copy-pasted from tmmpy_core and modified) code is used to
    allow speeding up the calculation utilizing the specific assumptions of the monte carlo run (e.g. the interfaces
    for each run are constant, so they can be pre processed).
    :param lmb: excitation wavelength
    :param ang: angle of incidence (in rad)
    :param bs: bragg stack object ("BraggStack")
    :param res: spatial resolution per layer
    :param pol: polarisation "s" or "p"
    :param itr: number of monte carlo iterations
    :param std: standard deviation of the gaussian distribution from which the d_depths are draw
    :return: vector of spatial points, vector of relative irradiance at each point in the active layers
    """
    pre = pre_process(bs, lmb, ang)
    active_idxs = bs.get_active_idxs()
    res_vec = [[i, res] for i in active_idxs]
    # Allocate beta matrix for speed.
    beta_matrix = array([[exp(1j * 0), 0], [0, exp(- 1j * 0)]], dtype=complex)

    def inner_func(beta, i, out):
        beta_matrix[0, 0] = exp(1j * beta)
        beta_matrix[1, 1] = exp(-1j * beta)
        return eval_irr(i, pre, np.dot(beta_matrix, out['E'][i]), pol)

    def scan_func(data, out):
        pre['d'] = data['d']
        pts, scan = tc._scan_stack(pre, lambda beta, i: inner_func(beta, i, out), res_vec)
        return scan

    # Calculate bragg points Monte Carlo style.
    result = solve_tmm_mc(bs, lmb, pol, ang, {'E': eval_E_coherent, 'irr_scan': scan_func}, itr, std, std_map)
    irr_mc = np.mean(np.array(result['irr_scan']), axis=0)
    # Calculate reference points.
    bs_ref = construct_reference_stack(bs, np.sum([pre['d'][idx] for idx in active_idxs]))
    irr_ref = solve_tmm(bs_ref, lmb, pol, ang, seq=True)['T'][0]
    pts, _ = tc._scan_stack(pre, lambda x, y: 0, res_vec)
    return pts, irr_mc, irr_ref


# endregion

# region IrrResult mc std_map large lmb range

params_mc = default_irr_params(lmbds=lmbds_small, angs=thetas, lmbs=lmbs)
params_mc["itr"] = itr
params_mc["std_low"] = std_low
params_mc["std_high"] = std_high
std_map = {bs.low.name: params_mc["std_low"], bs.high.name: params_mc["std_high"]}
wd_eval = "_".join(["mc", "lmbs"])
os.makedirs(wd, exist_ok=True)
irr_result_mc = run_irr_calculation(name=wd_eval, bs=bs, param_map=params_mc, wd=wd, calc=calc_irr_data_mc, res=res)

# endregion

# region IrrResult mc std_map for large lmbd range

params_mc = default_irr_params(lmbds=lmbds, angs=thetas, lmbs=lmbs_small)
params_mc["itr"] = itr
params_mc["std_low"] = std_low
params_mc["std_high"] = std_high
std_map = {bs.low.name: params_mc["std_low"], bs.high.name: params_mc["std_high"]}
wd_eval = "_".join(["mc", "lmbds"])
os.makedirs(wd, exist_ok=True)
irr_result_mc = run_irr_calculation(name=wd_eval, bs=bs, param_map=params_mc, wd=wd, calc=calc_irr_data_mc, res=res)

# endregion
