import matplotlib.pyplot as plt
from bragg_objects import *


def sample1():
    rem_result = load("/scratch_remote/rem_lmbd_scan_n40_ul25_itr50000_std1/rem_result.json")
    lmbds = get_param(rem_result, "lmbd")
    irrs = get_param(rem_result, "irr")
    # Setup mesh.
    X, Y = np.meshgrid(irrs, lmbds)
    Z = np.reshape(rem_result.get_ucqy_brg() * 100, X.shape)
    # Plot stuff.
    plt.pcolor(Y, X, Z, cmap='jet')
    CS = plt.contour(Y, X, Z, colors='k')
    plt.clabel(CS, inline=1, fontsize=10, fmt='%3.1f')
    # plt.ylim(0, 5000)
    # plt.colorbar()
    # Mark max.
    idx = Z == np.max(Z)
    max_str = "{:.2f}% @ ({:.0f},{:.1f})".format(Z[idx][0], Y[idx][0], X[idx][0])
    plt.plot(Y[idx], X[idx], ls='none', marker='o', color='white', label=max_str)  # cmap='RdBu_r')
    plt.legend()


def sample2(irr):
    rem_result = load("/scratch_remote/rem_mc_test/rem_n1_nscan_ulscan_{}wm2_itr50000_std1/rem_result.json".format(irr))
    # This object is damaged, values not present, so load them manually instead.
    n_highs = np.arange(1.6, 4.1, 0.1)
    num_uls = np.linspace(1, 50, 50)
    # Setup mesh.
    X, Y = np.meshgrid(num_uls, n_highs)
    Z = np.reshape(rem_result.get_ucqy_brg() * 100, X.shape)
    # Plot stuff.
    plt.pcolor(Y, X, Z, cmap='jet')
    CS = plt.contour(Y, X, Z, colors='k')
    plt.clabel(CS, inline=1, fontsize=10, fmt='%3.1f', lw=0.5, alpha=0.5)
    # plt.colorbar()
    plt.title("irr = {} W/m2".format(irr))
    # Mark max.
    idx = Z == np.max(Z)
    max_str = "{:.2f}% @ ({:.1f},{:.0f})".format(Z[idx][0], Y[idx][0], X[idx][0])
    plt.plot(Y[idx], X[idx], ls='none', marker='o', color='black', label=max_str)  # cmap='RdBu_r')
    plt.legend()


def sample2all():
    plt.figure(figsize=(7, 5))
    plt.subplot(2, 3, 1)
    sample2(30)
    plt.subplot(2, 3, 2)
    sample2(100)
    plt.subplot(2, 3, 3)
    sample2(500)
    plt.subplot(2, 3, 4)
    sample2(1000)
    plt.subplot(2, 3, 5)
    sample2(5000)
    plt.subplot(2, 3, 6)
    sample2(10000)
    plt.tight_layout()


sample2all()
plt.show()
