from dos_core import *

import matplotlib.pyplot as plt


def make_band_plot_ky(args, step=None, omega_max=None, ls=None, modes=None):
    step = 1e-3 if step is None else step
    # Plot for kx = 0.
    ks, result_low = make_band_plot(args, [0, 2.1], 'y', 0 + step / 2.0, step, omega_max, ls, modes)
    # Plot for kx = 0.5.
    _, result_high = make_band_plot(args, [0, 2.1], 'y', 0.5 - step / 2.0, step, omega_max, ls, modes)
    # Plot shading.
    for pol in result_low.ws:
        color = "red" if pol == "te" else "blue"
        for i in range(len(result_low.ws[pol][0])):
            plt.fill_between(ks, result_low.ws[pol][:, i], result_high.ws[pol][:, i], color=color, alpha=0.2)


def make_band_plot_kxky(args, step=None, omega_max=None, ls=None, modes=None):
    step = 1e-3 if step is None else step
    # Plot for kx = 0.
    ax = plt.subplot(1, 3, 1)
    plt.title("kx = 0")
    make_band_plot(args, [0, 2.1], 'y', 0 + step / 2.0, step, omega_max, ls, modes)
    plt.xticks([0.5, 1.0, 1.5])
    ax.invert_xaxis()
    # Plot for ky = 0.
    plt.subplot(1, 3, 2)
    plt.title("ky = 0")
    make_band_plot(args, [0, 0.5], 'x', 0 + step / 2.0, step, omega_max, ls, modes)
    plt.yticks([])
    plt.ylabel("")
    plt.xticks([0.1, 0.2, 0.3, 0.4])
    # Plot for kx = 0.5.
    plt.subplot(1, 3, 3)
    plt.title("kx = 0.5")
    make_band_plot(args, [0, 2.1], 'y', 0.5 - step / 2.0, step, omega_max, ls, modes)
    plt.yticks([])
    plt.ylabel("")
    plt.xticks([0.5, 1.0, 1.5])
    plt.subplots_adjust(wspace=0)


def make_band_plot(args, lims, axis, pt, step=None, omega_max=None, ls=None, modes=None, cache=None):
    wd = "../tmp_test"
    step = 1e-3 if step is None else step
    omega_max = 1.2 if omega_max is None else omega_max
    # Calculate data if needed.
    key = "band_calc"
    grid = MpbGrid1D(lims[0], lims[1], step, axis, pt)
    result = cache
    if result is None:
        mpb_input = MpbInput(args, grid, field=False)
        run_mpb_calculation(key, mpb_input, agg=agg_mpb_results, wd=wd)
        result = load(os.path.join(wd, key, "mpb_result.json"))
    # Plot stuff.
    ks = result.__dict__["k{}".format(axis)]
    for pol in result.ws if modes is None else modes:
        color = "red" if pol == "te" else "blue"
        plt.plot(ks, result.ws[pol], ls='-' if ls is None else ls, color=color)
    plt.xlim(min(ks), max(ks))
    plt.ylim(0, omega_max)
    plt.ylabel("$\omega$ [2$\pi$c/a]")
    plt.xlabel("k{} [2$\pi$/a]".format(axis))
    return ks, result


def plot_dos_result(dos_result, key, modes=None, rel=True, fltr=None, flip=True, cmap=None, ls=None, w_min=None):
    data = assemble_dos_data(dos_result, key, modes, rel, flip, fltr)
    if "ldos" in key:
        plot_ldos(dos_result.w_bins, data, cmap, w_min)
    else:
        plot_dos(dos_result.w_bins, data, ls, w_min)


def plot_ldos(w_bins, data, cmap=None, w_min=None):
    if w_min is not None:
        idx_min = np.argmin(abs(w_bins-w_min))
        w_bins = w_bins[idx_min:]
        data = data[idx_min:, :]
    x = np.linspace(-0.5, 0.5, data.shape[1])
    X, Y = np.meshgrid(x, w_bins)
    plt.pcolor(X, Y, data, cmap='RdBu_r' if cmap is None else cmap)
    plt.xlim(-0.5, 0.5)
    plt.ylim(min(w_bins), max(w_bins))
    plt.xlabel("x [a]")
    plt.ylabel("$\omega$ [2$\pi$c/a]")
    plt.colorbar()
    plt.tight_layout()


def plot_dos(w_bins, data, ls=None, w_min=None):
    if w_min is not None:
        idx_min = np.argmin(abs(w_bins-w_min))
        w_bins = w_bins[idx_min:]
        data = data[idx_min:]
    plt.plot(w_bins, data, ls='-' if ls is None else ls)
    plt.xlim(min(w_bins), max(w_bins))
    plt.xlabel("$\omega$ [2$\pi$c/a]")
    plt.ylabel("DOS [a.u.]")
    plt.ticklabel_format(style='sci', axis='y', scilimits=(0, 0))
